;===========================================================================================
; Bmp_in1.asm  Fasm Flat assembler
; Charger un fichier binaire bitmap 16M de couleurs en mmoire et l'afficher  l'cran
; Le fichier binaire bitmap est intgr au fichier .exe
; Programm par AsmGges France
;===========================================================================================

format PE GUI 4.0
entry start

; Inclusion des macros/quates/structures ncessaires au programme
; ----------------------------------------------------------------
  include 'win32a.inc'
  include 'macro\if.inc'

  DIB_RGB_COLORS = 0h

;===========================================================================================
; Dbut du code programme
; -----------------------
section '.code' code readable executable

start:
    ; Obtenir le handle du programme
    ; ------------------------------
    invoke GetModuleHandle,0
    mov [wc.hInstance],eax

    ; Chargement d'une icne spcifique
    ; ---------------------------------
    invoke LoadIcon,[wc.hInstance],IDI_FRANCE
    mov [wc.hIcon],eax

    ; Chargement d'un curseur systme
    ; -------------------------------
    invoke LoadCursor,0,IDC_ARROW
    mov [wc.hCursor],eax

    ; Obtenir la largeur en pixels de l'cran
    ; ---------------------------------------
    invoke GetSystemMetrics,SM_CXSCREEN
    sub eax,[WinW]
    shr eax,1
    mov [WinX],eax

    ; Valider la structure WNDCLASS
    ; -----------------------------
    invoke RegisterClass,wc

    ; Cration de la fenetre de l'application
    ; ---------------------------------------
    invoke CreateWindowEx,0,ClassName,Titre,WS_CAPTION+WS_POPUP+WS_SYSMENU+\
                          WS_MINIMIZEBOX,[WinX],[WinY],[WinW],[WinH],0,0,[wc.hInstance],0
    mov [hWnd],eax

    ; Affiche et rafraichit la fenetre
    ; --------------------------------
    invoke ShowWindow,[hWnd],SW_SHOWDEFAULT
    invoke UpdateWindow,[hWnd]

 ; Boucle des messages
 ; -------------------
 MessageLoop:
    invoke GetMessage,msg,0,0,0
    or eax,eax
    jz ExitProgram
    invoke TranslateMessage,msg
    invoke DispatchMessage,msg
    jmp MessageLoop

 ExitProgram:
    ; Fermeture de l'application
    ; --------------------------
    invoke ExitProcess,[msg.wParam]

;===========================================================================================
; Procdure principale Windows
;-------------------------------------------------------------------------------------------
proc WindowProc uses ebx esi edi, @hWnd, @uMsg, @wParam, @lParam

    .if [@uMsg] = WM_CLOSE
        ; Boite  message fin d'application
        ; ---------------------------------
        invoke MessageBox,[@hWnd],MsgEnd,ClassName,MB_OK+MB_ICONINFORMATION

        ; Supprime l'objet bitmap cre
        ; ----------------------------
        invoke DeleteObject,[hBitmap]

        ; Fin de l'application
        ; --------------------
        invoke PostQuitMessage,0

    .elseif [@uMsg] = WM_CREATE

        ; Cration d'un dispositif bitmap indpendant
        ; -------------------------------------------
        mov ebx,ptBitmap                           ;addr BITMAPINFOHEADER
        invoke CreateDIBSection,0,ebx,DIB_RGB_COLORS,ppvBits,0,0
        mov [hBitmap],eax

        ; Copie le bitmap en mmoire
        ; --------------------------
        mov eax,ptBitmap+40                        ;addr src BitmapImage
        stdcall MemCopy,eax,[ppvBits],[ebx+20]     ;BITMAPINFOHEADER.biSizeImage

    .elseif [@uMsg] = WM_PAINT

        ; Dbut dessin/criture
        ; ---------------------
        invoke BeginPaint,[@hWnd],ps
        mov [ps.hdc],eax

        ; Cration d'un contexte de dispositif mmoire compatible
        ; -------------------------------------------------------
        invoke CreateCompatibleDC,[ps.hdc]
        mov [hMemDC],eax

        ; Choisit un objet suivant le contexte de dispositif hmdc
        ; -------------------------------------------------------
        invoke SelectObject,eax,[hBitmap]

        ; Copie le bitmap de la mmoire vers la fenetre
        ; ---------------------------------------------
        mov ebx,ptBitmap                           ;addr BITMAPINFOHEADER
        invoke BitBlt,[ps.hdc],84,18,[ebx+4],[ebx+8],[hMemDC],0,0,SRCCOPY    ;196

        ; Supprime le contexte de dispositif hMemDC
        ; -----------------------------------------
        invoke DeleteDC,[hMemDC]

        ; Fin dessin/criture
        ; -------------------
        invoke EndPaint,[@hWnd],ps

    .else
        ; Traitement par dfaut des autres messages Windows
        ; ------------------------------------------------
        invoke DefWindowProc,[@hWnd],[@uMsg],[@wParam],[@lParam]
        ret

    .endif

    mov eax,0
    ret
endp

;===========================================================================================
; @AddrSrc  = adresse source de la zone mmoire  copier
; @AddrDest = adresse de destination de la copie
; @Size     = nombre d'octets  copier
;-------------------------------------------------------------------------------------------
proc MemCopy @AddrSrc, @AddrDest, @Size

    cld
    mov esi,[@AddrSrc]
    mov edi,[@AddrDest]
    mov ecx,[@Size]
    shr ecx,2
    rep movsd
    mov ecx,[@Size]
    and ecx,3
    rep movsb

    ret
endp

;===========================================================================================
; Dfinitions des datas chaines / variables initialises ou non initialises / structures
;-------------------------------------------------------------------------------------------
section '.data' data readable writeable

  ptBitmap FILE 'res\Fyeah.bmp':14                   ;atteindre adresse BITMAPINFOHEADER
  Titre      db ' Bitmap 16M from memory',0
  ClassName  db ' AsmGges Win32',0
  MsgEnd     db ' AsmGges France @2006  ',0
  align 4

  WinX       dd 0
  WinY       dd 100
  WinW       dd 300
  WinH       dd 200

  hWnd       dd ?
  hBitmap    dd ?
  hMemDC     dd ?
  ppvBits    dd ?

  msg        MSG
  wc         WNDCLASS  0,WindowProc,0,0,NULL,NULL,NULL,COLOR_WINDOW+1,NULL,ClassName
  ps         PAINTSTRUCT

;===========================================================================================
; Inclusion des fichiers .dll contenant les fonctions Windows utilises
;-------------------------------------------------------------------------------------------
section '.idata' import data readable writeable

  library kernel32,'KERNEL32.DLL',\
            user32,'USER32.DLL',\
             gdi32,'GDI32.DLL'

  ; Apis Windows dans fichiers dlls
  ; -------------------------------
  include 'apia\kernel32.inc'
  include 'apia\user32.inc'
  include 'apia\gdi32.inc'

;===========================================================================================
; Inclusion des icnes / curseurs / bitmap
;-------------------------------------------------------------------------------------------
section '.rsrc' resource data readable

  directory RT_ICON,icons,\
            RT_GROUP_ICON,group_icons

  IDI_FRANCE = 17

  resource icons,\
           1,LANG_NEUTRAL,icon_data

  resource group_icons,\
           IDI_FRANCE,LANG_NEUTRAL,main_icon

  icon main_icon,icon_data,'res\france.ico'

;===========================================================================================

